@@ -15,17 +15,25 @@ import re
 from os.path import expanduser
 
 
-def is_valid_token(token, config):
-    headers = {"Content-type": "application/json", "Authorization": "{}".format(token)}
-    kbapi_url = config.get_kbapi_url()
-    url = kbapi_url + "/verify"
-    for i in range(6):
-        r = requests.get(url, headers=headers)
-        if r.status_code == 200:
-            return True
-        else:
-            r.raise_for_status()  # Some unexpected status that's not caught
-        time.sleep(2 ** i)  # exponential backoff
+info = click.echo
+
+
+def error(msg):
+    raise click.ClickException(msg)
+
+
+def warning(msg):
+    click.echo(msg, err=True)
+
+
+def no_info(_):
+    pass
+
+
+def quiet_output(quiet):
+    global info
+    if quiet:
+        info = no_info
 
 
 def get_config(config_path="~/.config/tuxbuild/config.ini"):
@@ -37,17 +45,16 @@ def get_config(config_path="~/.config/tuxbuild/config.ini"):
         configparser.NoSectionError,
         tuxbuild.exceptions.TokenNotFound,
     ) as e:
-        raise click.ClickException(e)
+        error(e)
 
 
 def get_auth_token(config):
-    token = config.get_auth_token()
     try:
-        is_valid_token(token, config)
+        config.check_auth_token()
+        return config.get_auth_token()
     except Exception as e:
         print(e)
-        raise (click.ClickException("😔 Invalid Token"))
-    return token
+        error("😔 Invalid Token")
 
 
 def get_default_buildset():
@@ -57,11 +64,11 @@ def get_default_buildset():
 
 def wait_for_state_change(build_object, state_items):
     for item in state_items:
-        click.echo(item.get("message") + str(build_object))
+        info(item.get("message") + str(build_object))
         try:
             build_object.wait_on_status(item.get("state"))
         except tuxbuild.exceptions.Timeout as e:
-            click.echo(
+            warning(
                 click.style(
                     "🔧 Tuxbuild Error ({}): Please resubmit this build", fg="white"
                 ).format(str(e))
@@ -88,7 +95,7 @@ def wait_for_object(build_object):
     # Check for build result
     elif build_object.tuxbuild_status != "complete":
         status_message = build_object.status_message
-        click.echo(
+        warning(
             click.style("🔧 Tuxbuild Error ({}): ", fg="white").format(status_message)
             + str(build_object)
         )
@@ -103,7 +110,7 @@ def wait_for_object(build_object):
             error_message = f"({errors_count} errors)"
         if status_message:
             error_message += f" with status message '{status_message}'"
-        click.echo(
+        warning(
             click.style("👹 Fail {}: ", fg="bright_red", bold=True).format(error_message)
             + str(build_object)
         )
@@ -111,7 +118,7 @@ def wait_for_object(build_object):
     elif build_object.build_status == "pass":
         warnings_count = build_object.warnings_count
         if warnings_count == 0:
-            click.echo(
+            info(
                 click.style(
                     "🎉 Pass ({} warnings): ".format(warnings_count),
                     fg="green",
@@ -120,7 +127,7 @@ def wait_for_object(build_object):
                 + str(build_object)
             )
         elif warnings_count == 1:
-            click.echo(
+            warning(
                 click.style(
                     "👾 Pass ({} warning): ".format(warnings_count),
                     fg="yellow",
@@ -129,7 +136,7 @@ def wait_for_object(build_object):
                 + str(build_object)
             )
         else:
-            click.echo(
+            warning(
                 click.style(
                     "👾 Pass ({} warnings): ".format(warnings_count),
                     fg="yellow",
@@ -162,7 +169,7 @@ def cli():
     help="Kernel kconfig arguments (may be specified multiple times)",
 )
 @click.option(
-    "--toolchain", required=True, help="Toolchain [gcc-8|gcc-9|clang-8|clang-9]",
+    "--toolchain", required=True, help="Toolchain [gcc-8|gcc-9|clang-8|clang-9]"
 )
 @click.option(
     "--kconfig-allconfig",
@@ -178,6 +185,13 @@ def cli():
     help="Write json build status out to a named file path",
     type=click.File("w", encoding="utf-8"),
 )
+@click.option(
+    "-q",
+    "--quiet",
+    default=False,
+    is_flag=True,
+    help="Supress all informational output; prints only the download URL for the build",
+)
 def build(
     git_repo,
     git_ref,
@@ -186,21 +200,24 @@ def build(
     toolchain,
     kconfig_allconfig=None,
     json_out=None,
+    quiet=False,
 ):
+    quiet_output(quiet)
+
     auth = get_config()
     token = get_auth_token(auth)
     kbapi_url = auth.get_kbapi_url()
     build = tuxbuild.build.Build(
-        git_repo,
-        git_ref,
-        target_arch,
-        kconfig,
-        toolchain,
-        token,
-        kbapi_url,
-        kconfig_allconfig,
+        git_repo=git_repo,
+        git_ref=git_ref,
+        target_arch=target_arch,
+        kconfig=kconfig,
+        toolchain=toolchain,
+        token=token,
+        kbapi_url=kbapi_url,
+        kconfig_allconfig=kconfig_allconfig,
     )
-    click.echo("Building Linux Kernel {} at {}".format(git_repo, git_ref))
+    info("Building Linux Kernel {} at {}".format(git_repo, git_ref))
     try:
         build.build()
     except tuxbuild.exceptions.BadRequest as e:
@@ -208,6 +225,8 @@ def build(
     build_result = wait_for_object(build)
     if json_out:
         json_out.write(json.dumps(build.status, sort_keys=True, indent=4))
+    if quiet:
+        print(build.build_data)
     if not build_result:
         sys.exit(1)
 
@@ -245,7 +264,7 @@ def get_tux_config(url, retries=3, sleep=3):
     help="Kernel kconfig arguments (may be specified multiple times)",
 )
 @click.option(
-    "--toolchain", required=False, help="Toolchain [gcc-8|gcc-9|clang-8|clang-9]",
+    "--toolchain", required=False, help="Toolchain [gcc-8|gcc-9|clang-8|clang-9]"
 )
 @click.option(
     "--kconfig-allconfig",
@@ -261,6 +280,13 @@ def get_tux_config(url, retries=3, sleep=3):
     help="Write json build status out to a named file path",
     type=click.File("w", encoding="utf-8"),
 )
+@click.option(
+    "-q",
+    "--quiet",
+    default=False,
+    is_flag=True,
+    help="Supress all informational output; prints only the download URLs for the builds",
+)
 def build_set(
     tux_config,
     set_name,
@@ -271,7 +297,10 @@ def build_set(
     kconfig_allconfig=None,
     toolchain=None,
     json_out=None,
+    quiet=None,
 ):
+    quiet_output(quiet)
+
     params = {}
     params["git_repo"] = git_repo
     params["git_ref"] = git_ref
@@ -289,7 +318,7 @@ def build_set(
         with open(tux_config, "r") as f:
             tux_config_contents = yaml.safe_load(f)
 
-    click.echo("Building Linux Kernel build set {}".format(set_name))
+    info("Building Linux Kernel build set {}".format(set_name))
     # Find build named set_name
     build_list = None
     for bs in tux_config_contents.get("sets"):
@@ -302,25 +331,17 @@ def build_set(
 
     # Build not found in config
     if not build_list:
-        raise click.ClickException(
-            "No set named {} found in {}".format(set_name, tux_config)
-        )
+        error("No set named {} found in {}".format(set_name, tux_config))
 
     # Create build objects. Do this first to take advantage of argument validation
     # before submitting builds
     build_objects = []
     for b in build_list:
-        for param in [
-            "git_repo",
-            "git_ref",
-            "target_arch",
-            "kconfig",
-            "toolchain",
-        ]:
+        for param in ["git_repo", "git_ref", "target_arch", "kconfig", "toolchain"]:
             if params[param]:
                 b[param] = params[param]
             elif param not in b:
-                raise click.ClickException(
+                error(
                     "No {} found in command line or in the set {} in {}: {}".format(
                         param, set_name, tux_config, b
                     )
@@ -335,14 +356,14 @@ def build_set(
 
         build_objects.append(
             tuxbuild.build.Build(
-                b["git_repo"],
-                b["git_ref"],
-                b["target_arch"],
-                b["kconfig"],
-                b["toolchain"],
-                token,
-                kbapi_url,
-                b["kconfig_allconfig"],
+                git_repo=b["git_repo"],
+                git_ref=b["git_ref"],
+                target_arch=b["target_arch"],
+                kconfig=b["kconfig"],
+                toolchain=b["toolchain"],
+                token=token,
+                kbapi_url=kbapi_url,
+                kconfig_allconfig=b["kconfig_allconfig"],
             )
         )
 
@@ -365,6 +386,10 @@ def build_set(
     if json_out:
         json_out.write(json.dumps(build_set.status_list, sort_keys=True, indent=4))
 
+    if quiet:
+        for build in build_set.build_objects:
+            print(build.build_data)
+
     # If any of the builds did not pass, exit with exit code of 1
     if False in [result._result for result in build_results]:
         sys.exit(1)
